#
#  PyWeeWxClient - Python library to access the weather observations of a WeeWX station
# 
#  Copyright (c) 2021 Steve Cross <flip@foxhollow.cc>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 2.1 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
#

import urllib
import urllib.request
import yaml

class Generation:
    """Details about when and what generated these observations"""
    def __init__(self, inObject: any):
        """Time the observations were made"""
        self.time: str = inObject["time"]

        """Version of WeeWX that made the observations"""
        self.generator: str = inObject["generator"]


class Station:
    def __init__(self, inObject: any):
        self.location: str = inObject["location"]
        self.latitude: float = inObject["latitude"]
        self.longitude: float = inObject["longitude"]
        self.altitude: float = inObject["altitude (meters)"]
        self.link: str = inObject["link"]
    

class AggregateStat:
    def __init__(self, stat: any):
        self.value: float = stat["value"]
        self.units: str = stat["units"]
        self.at: str = stat["at"]

class AggregateStats:
    def __init__(self, inObject: any):
        self.max_temperature: AggregateStat = AggregateStat(inObject["max temperature"])
        self.min_temperature: AggregateStat = AggregateStat(inObject["min temperature"])
        self.max_dewpoint: AggregateStat = AggregateStat(inObject["max dewpoint"])
        self.min_dewpoint: AggregateStat = AggregateStat(inObject["min dewpoint"])
        self.max_humidity: AggregateStat = AggregateStat(inObject["max humidity"])
        self.min_humidity: AggregateStat = AggregateStat(inObject["min humidity"])
        self.max_barometer: AggregateStat = AggregateStat(inObject["max barometer"])
        self.min_barometer: AggregateStat = AggregateStat(inObject["min barometer"])
        self.max_wind_speed: AggregateStat = AggregateStat(inObject["max wind speed"])
        self.max_wind_gust: AggregateStat = AggregateStat(inObject["max wind gust"])
        self.max_rain_rate: AggregateStat = AggregateStat(inObject["max rain rate"])
        self.rain_total: CurrentStat = CurrentStat(inObject["rain total"])
        self.max_inside_temperature: AggregateStat = AggregateStat(inObject["max inside temperature"])
        self.min_inside_temperature: AggregateStat = AggregateStat(inObject["min inside temperature"])


class CurrentStat:
    def __init__(self, stat):
        self.value: float = stat["value"]
        self.units: str = stat["units"]

class CurrentStats:
    def __init__(self, inObject: any):
        self.temperature: CurrentStat = CurrentStat(inObject["temperature"])
        self.dewpoint: CurrentStat = CurrentStat(inObject["dewpoint"])
        self.humidity: CurrentStat = CurrentStat(inObject["humidity"])
        self.heat_index: CurrentStat = CurrentStat(inObject["heat index"])
        self.barometer: CurrentStat = CurrentStat(inObject["barometer"])
        self.wind_speed: CurrentStat = CurrentStat(inObject["wind speed"])
        self.wind_gust: CurrentStat = CurrentStat(inObject["wind gust"])
        self.wind_chill: CurrentStat = CurrentStat(inObject["wind chill"])
        self.rain_rate: CurrentStat = CurrentStat(inObject["rain rate"])
        self.inside_temperature: CurrentStat = CurrentStat(inObject["inside temperature"])


class WeeWxWeather:
    """Weather observations as read from a WeeWX station"""

    def __init__(self, inObject: any):
        self.station: Station = Station(inObject["station"])
        self.generation: Generation = Generation(inObject["generation"])
        self.current: CurrentStats = CurrentStats(inObject["current"])
        self.day: AggregateStats = AggregateStats(inObject["day"])
        self.week: AggregateStats = AggregateStats(inObject["week"])
        self.month: AggregateStats = AggregateStats(inObject["month"])
        self.year: AggregateStats = AggregateStats(inObject["year"])
    

def fetch(url: str) -> WeeWxWeather:
    """Fetch the most recent weather observations

    :param url: full url to the json file where the observations should be read from
    """
    try:
        response = urllib.request.urlopen(url)
    except urllib.error.HTTPError:
        return None

    data = yaml.load(response.read(), Loader=yaml.loader.SafeLoader)

    return WeeWxWeather(data);

weather = fetch("https://weather.foxhollow.cc/weewx/weewx.json")

if weather == None:
    print("Unable to fetch json")
else:
    print(yaml.dump(weather))
